/* $Id: phtoelm.def,v 1.5 1994/02/24 15:03:05 a904209 Exp a904209 $
*/
enter(" ","Q",NULL);
enter("p","P","PY","PZ",NULL);
enter("t","T","TY","TZ",NULL);
enter("k","K","KY","KZ",NULL);
enter("b","B","BY","BZ",NULL);
enter("d","D","DY","DZ",NULL);
enter("g","G","GY","GZ",NULL);
enter("m","M",NULL);
enter("n","N",NULL);
enter("N","NG",NULL);
enter("f","F",NULL);
enter("T","TH",NULL);
enter("s","S",NULL);
enter("S","SH",NULL);
enter("h","H",NULL);
enter("v","V","QQ","VZ",NULL);
enter("D","DH","QQ","DI",NULL);
enter("z","Z","QQ","ZZ",NULL);
enter("Z","ZH","QQ","ZI",NULL);
enter("tS","CH","CI",NULL);
enter("dZ","J","JY","QQ","JZ",NULL);
enter("l","L",NULL);
enter("r","R",NULL);
enter("R","RX",NULL);
enter("w","W",NULL);
enter("j","Y",NULL);
enter("I","I",NULL);
enter("e","E",NULL);
enter("&","AA",NULL);
enter("V","U",NULL);
enter("0","O",NULL);
enter("U","OO",NULL);
enter("@","A",NULL);
enter("i","EE",NULL);
enter("3","ER",NULL);
enter("A","AR",NULL);
enter("O","AW",NULL);
enter("u","UU",NULL);
enter("eI","AI","AJ",NULL);
enter("aI","IE","IF",NULL);
enter("oI","OI","OJ",NULL);
enter("aU","OU","OV",NULL);
enter("@U","OA","OB",NULL);
enter("I@","IA","IB",NULL);
enter("e@","AIR","AS",NULL);
enter("U@","OOR","OP",NULL);
enter("O@","OR","OS",NULL);
